# Falken Dashboard

To visualize data sent to the [Falken service](../service/README.md) for
debugging purposes, a web dashboard can be run on your machine by following
the steps below.

## Install Dependencies

To build and run the dashboard you need to install the following dependencies:

 * [Node.js](https://nodejs.org) 12.14.1 or newer.

## Install Package Dependencies

After installing Node.js and adding `npm` and `npx` command line tools to your
path run the following command to install all package dependencies:

```
npm install
```

## Build and Launch the Dashboard

Run the following:

```
npm run start
```

The above command performs the following operations:

 * Downloads additional build tools.
 * Generates [gRPC](https://github.com/grpc/grpc-web) JavaScript interface for
   Falken's service.
 * Packages the dashboard's [JavaScript modules](src/).
 * Launches a [proxy server](https://github.com/improbable-eng/grpc-web/tree/master/go/grpcwebproxy)
   to translate gRPC-web protocol requests from the client to gRPC for the
   service.
 * Launches a [web server](https://webpack.js.org/configuration/dev-server/)
   on port 8000 (see [package.json](pacakge.json)) to serve the dashboard.

## Login

Navigate to [http://localhost:8000](http://localhost:8000) and enter
your project ID and API key from the `falken_config.json` generated by the
[service](../service/README.md#launch-the-service).

## View an Episode

After [logging in](#login), and creating some brains and sessions with an
environment like [Hello Falken](environments/unity/demos/Assets/HelloFalken),
you need to do the following to view data for each episode:

 * Select a brain from **Brain** drop down at the top of the page. This will
   load the available sessions.
 * Select a session from the **Session** drop down at the top of the page. This
   will load the available episodes.
 * Select an episode from the **Episode** drop down at the top of the page which
   should populate the **Attributes** and, if the session has entities with
   transforms (positions / rotations), the **Scene View**. All episodes are
   listed in reverse chronological order (newest first).

## Viewing Attributes (Observations / Actions)

To view an attribute (observation / action) in more detail, use the left mouse
button to select one or more graphs from the **Attributes** section and they
will be overlaid in the **Timeline** section of the page. You can hover the
mouse over graphs to see the value of an attribute at each point in time.

### Zoom into Timeline

 * Hover the mouse cursor over the start of the zoom area on the timeline.
 * Hold the shift key and click and hold the left mouse button.
 * Drag the mouse cursor to the end of the zoom area.
 * Release the left mouse button to zoom into the selected area.
 * To reset the zoom, click the **Reset Zoom** button on the top right of the
   **Timeline** view.

## Scene View

If you provided transforms (positions / rotations) of entities and feelers,
traces of each entity and their feelers are rendered in the **Scene View**.

### Navigation

The following interactions change the camera position in the view:

 * **Zoom in** by hovering the mouse cursor over the scene view and either:
   * Scrolling the mouse wheel up.
   * Holding the middle mouse button and moving the mouse cursor up.
 * **Zoom out** by hovering the mouse cursor over the scene view and either:
   * Scrolling the mouse wheel down.
   * Holding the middle mouse button and moving the mouse cursor down.
 * **Rotate pitch down** by hovering the mouse cursor over the scene view,
   holding the left mouse button and moving the mouse cursor down.
 * **Rotate pitch up** by hovering the mouse cursor over the scene view,
   holding the left mouse button and moving the mouse cursor up.
 * **Rotate yaw left** by hovering the mouse cursor over the scene view, holding
   the left mouse button and moving the mouse cursor left.
 * **Rotate yaw right** by hovering the mouse cursor over the scene view,
   holding the left mouse button and moving the mouse cursor right.
 * **Pan left** by hovering the mouse cursor over the scene view,
   holding the right mouse button and moving the mouse cursor left.
 * **Pan right** by hovering the mouse cursor over the scene view,
   holding the right mouse button and moving the mouse cursor right
 * **Pan up** by hovering the mouse cursor over the scene view,
   holding the right mouse button and moving the mouse cursor up.
 * **Pan down** by hovering the mouse cursor over the scene view,
   holding the right mouse button and moving the mouse cursor down.

### View Progression

To view how entities and the player's feelers have moved over time:

 * Select one or more a graph in **Attributes** view.
 * Move your mouse to the **Timeline** graph.
 * Click and hold the left mouse button and drag across the graph to observe
   entities moving in the **Scene View**.
